/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */

package com.yifu.cloud.plus.v1.yifu.salary.entity;

import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttribute;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;

/**
 * 统计-申报人员
 *
 * @author hgw
 * @date 2022-08-05 11:40:14
 */
@Data
@TableName("t_statistics_tax_declarer")
@Schema(description = "统计-申报人员")
public class TStatisticsTaxDeclarer {

	/**
	 * id
	 */
	@TableId(type = IdType.ASSIGN_ID)
	@ExcelProperty("id")
	private String id;
	/**
	 * 税务主体(封面抬头)
	 */
	@ExcelAttribute(name = "税务主体(封面抬头)", maxLength = 50)
	@Length(max = 50, message = "税务主体(封面抬头)不能超过50个字符")
	@ExcelProperty("税务主体(封面抬头)")
	private String invoiceTitle;
	/**
	 * 结算部门id
	 */
	@ExcelAttribute(name = "结算部门id", isNotEmpty = true, errorInfo = "结算部门id不能为空", maxLength = 32)
	@NotBlank(message = "结算部门id不能为空")
	@Length(max = 32, message = "结算部门id不能超过32个字符")
	@ExcelProperty("结算部门id")
	private String settleDepartId;
	/**
	 * 结算部门no
	 */
	@ExcelAttribute(name = "结算部门no", isNotEmpty = true, errorInfo = "结算部门no不能为空", maxLength = 50)
	@NotBlank(message = "结算部门no不能为空")
	@Length(max = 50, message = "结算部门no不能超过50个字符")
	@ExcelProperty("结算部门no")
	private String settleDepartNo;
	/**
	 * 结算部门name
	 */
	@ExcelAttribute(name = "结算部门name", isNotEmpty = true, errorInfo = "结算部门name不能为空", maxLength = 50)
	@NotBlank(message = "结算部门name不能为空")
	@Length(max = 50, message = "结算部门name不能超过50个字符")
	@ExcelProperty("结算部门name")
	private String settleDepartName;
	/**
	 * 申报月份
	 */
	@ExcelAttribute(name = "申报月份", isNotEmpty = true, errorInfo = "申报月份不能为空", maxLength = 6)
	@NotBlank(message = "申报月份不能为空")
	@Length(max = 6, message = "申报月份不能超过6个字符")
	@ExcelProperty("申报月份")
	private String declareMonth;
	/**
	 * 员工 ID
	 */
	@ExcelAttribute(name = "员工 ID", maxLength = 32)
	@Length(max = 32, message = "员工 ID不能超过32个字符")
	@ExcelProperty("员工 ID")
	private String empId;
	/**
	 * 员工姓名
	 */
	@ExcelAttribute(name = "员工姓名", maxLength = 50)
	@Length(max = 50, message = "员工姓名不能超过50个字符")
	@ExcelProperty("员工姓名")
	private String empName;
	/**
	 * 员工身份证号
	 */
	@ExcelAttribute(name = "员工身份证号", maxLength = 25)
	@Length(max = 25, message = "员工身份证号不能超过25个字符")
	@ExcelProperty("员工身份证号")
	private String empIdcard;
	/**
	 * 手机号码
	 */
	@ExcelAttribute(name = "手机号码", maxLength = 25)
	@Length(max = 25, message = "手机号码不能超过25个字符")
	@ExcelProperty("手机号码")
	private String phone;
	/**
	 * 人员状态(在职/临时/离职)
	 */
	@ExcelAttribute(name = "人员状态(在职/临时/离职)", maxLength = 5)
	@Length(max = 5, message = "人员状态(在职/临时/离职)不能超过5个字符")
	@ExcelProperty("人员状态(在职/临时/离职)")
	private String userStatus;
	/**
	 * 任职受雇从业类型(雇员/其他（劳务人员）)-目前为雇员
	 */
	@ExcelAttribute(name = "任职受雇从业类型(雇员/其他（劳务人员）)-目前为雇员", maxLength = 5)
	@Length(max = 5, message = "任职受雇从业类型(雇员/其他（劳务人员）)-目前为雇员不能超过5个字符")
	@ExcelProperty("任职受雇从业类型(雇员/其他（劳务人员）)-目前为雇员")
	private String userType;
	/**
	 * 任职受雇从业日期(计税月份)
	 */
	@ExcelAttribute(name = "任职受雇从业日期(计税月份)", maxLength = 20)
	@Length(max = 20, message = "任职受雇从业日期(计税月份)不能超过20个字符")
	@ExcelProperty("任职受雇从业日期(计税月份)")
	private String taxMonth;
	/**
	 * 单位id
	 */
	@ExcelAttribute(name = "单位id", maxLength = 32)
	@Length(max = 32, message = "单位id不能超过32个字符")
	@ExcelProperty("单位id")
	private String unitId;
	/**
	 * 单位no
	 */
	@ExcelAttribute(name = "单位no", maxLength = 50)
	@Length(max = 50, message = "单位no不能超过50个字符")
	@ExcelProperty("单位no")
	private String unitNo;
	/**
	 * 单位name
	 */
	@ExcelAttribute(name = "单位name", maxLength = 50)
	@Length(max = 50, message = "单位name不能超过50个字符")
	@ExcelProperty("单位name")
	private String unitName;
	/**
	 * 个税
	 */
	@ExcelAttribute(name = "个税")
	@ExcelProperty("个税")
	private BigDecimal salaryTax;
	/**
	 * 专项总金额
	 */
	@ExcelAttribute(name = "专项总金额")
	@ExcelProperty("专项总金额")
	private BigDecimal specialDeduMoney;

}
