/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */
package com.yifu.cloud.plus.v1.yifu.salary.vo;

import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.ExcelAttribute;
import com.yifu.cloud.plus.v1.yifu.common.core.vo.RowIndex;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.math.BigDecimal;

/**
 * 统计-全年一次性奖金
 *
 * @author hgw
 * @date 2022-08-05 11:40:15
 */
@Data
public class TStatisticsAnnualBonusVo extends RowIndex implements Serializable {

	/**
	 * 税务主体(封面抬头)
	 */
	@Length(max = 50, message = "税务主体(封面抬头) 不能超过50 个字符")
	@ExcelAttribute(name = "税务主体(封面抬头)", maxLength = 50)
	@Schema(description = "税务主体(封面抬头)")
	@ExcelProperty("税务主体(封面抬头)")
	private String invoiceTitle;
	/**
	 * 结算部门id
	 */
	@NotBlank(message = "结算部门id 不能为空")
	@Length(max = 32, message = "结算部门id 不能超过32 个字符")
	@ExcelAttribute(name = "结算部门id", isNotEmpty = true, errorInfo = "结算部门id 不能为空", maxLength = 32)
	@Schema(description = "结算部门id")
	@ExcelProperty("结算部门id")
	private String settleDepartId;
	/**
	 * 结算部门no
	 */
	@NotBlank(message = "结算部门no 不能为空")
	@Length(max = 50, message = "结算部门no 不能超过50 个字符")
	@ExcelAttribute(name = "结算部门no", isNotEmpty = true, errorInfo = "结算部门no 不能为空", maxLength = 50)
	@Schema(description = "结算部门no")
	@ExcelProperty("结算部门no")
	private String settleDepartNo;
	/**
	 * 结算部门name
	 */
	@NotBlank(message = "结算部门name 不能为空")
	@Length(max = 50, message = "结算部门name 不能超过50 个字符")
	@ExcelAttribute(name = "结算部门name", isNotEmpty = true, errorInfo = "结算部门name 不能为空", maxLength = 50)
	@Schema(description = "结算部门name")
	@ExcelProperty("结算部门name")
	private String settleDepartName;
	/**
	 * 申报月份（YYYYMM）
	 */
	@NotBlank(message = "申报月份（YYYYMM） 不能为空")
	@Length(max = 6, message = "申报月份（YYYYMM） 不能超过6 个字符")
	@ExcelAttribute(name = "申报月份（YYYYMM）", isNotEmpty = true, errorInfo = "申报月份（YYYYMM） 不能为空", maxLength = 6)
	@Schema(description = "申报月份（YYYYMM）")
	@ExcelProperty("申报月份（YYYYMM）")
	private String declareMonth;
	/**
	 * 员工 ID
	 */
	@Length(max = 32, message = "员工 ID 不能超过32 个字符")
	@ExcelAttribute(name = "员工 ID", maxLength = 32)
	@Schema(description = "员工 ID")
	@ExcelProperty("员工 ID")
	private String empId;
	/**
	 * 员工姓名
	 */
	@Length(max = 50, message = "员工姓名 不能超过50 个字符")
	@ExcelAttribute(name = "员工姓名", maxLength = 50)
	@Schema(description = "员工姓名")
	@ExcelProperty("员工姓名")
	private String empName;
	/**
	 * 员工身份证号
	 */
	@Length(max = 25, message = "员工身份证号 不能超过25 个字符")
	@ExcelAttribute(name = "员工身份证号", maxLength = 25)
	@Schema(description = "员工身份证号")
	@ExcelProperty("员工身份证号")
	private String empIdcard;
	/**
	 * 年终奖
	 */
	@ExcelAttribute(name = "年终奖")
	@Schema(description = "年终奖")
	@ExcelProperty("年终奖")
	private BigDecimal annualBonus;
	/**
	 * 年终奖单独税
	 */
	@ExcelAttribute(name = "年终奖单独税")
	@Schema(description = "年终奖单独税")
	@ExcelProperty("年终奖单独税")
	private BigDecimal tax;
	/**
	 * 单位id
	 */
	@Length(max = 32, message = "单位id 不能超过32 个字符")
	@ExcelAttribute(name = "单位id", maxLength = 32)
	@Schema(description = "单位id")
	@ExcelProperty("单位id")
	private String unitId;
	/**
	 * 单位no
	 */
	@Length(max = 50, message = "单位no 不能超过50 个字符")
	@ExcelAttribute(name = "单位no", maxLength = 50)
	@Schema(description = "单位no")
	@ExcelProperty("单位no")
	private String unitNo;
	/**
	 * 单位name
	 */
	@Length(max = 50, message = "单位name 不能超过50 个字符")
	@ExcelAttribute(name = "单位name", maxLength = 50)
	@Schema(description = "单位name")
	@ExcelProperty("单位name")
	private String unitName;

}
